from optimizer import solve_optimization
from landmarks_manager import generate_landmarks
from structs.landmarks import LandmarkTest
from structs.landmarks import Landmark
from structs.landmarktype import LandmarkType
from structs.preferences import Preferences, Preference
from fastapi import FastAPI, Query, Body
from typing import List

app = FastAPI()


# Assuming frontend is calling like this : 
#"http://127.0.0.1:8000/process?param1={param1}&param2={param2}"
@app.post("/optimizer_coords/{latitude}/{longitude}/{city_country}")
def main1(preferences: Preferences = Body(...), latitude: float = None, longitude: float = None, city_country: str = None) -> List[Landmark]:
    
    if preferences is None :
        raise ValueError("Please provide preferences in the form of a 'Preference' BaseModel class.")
    elif latitude is None and longitude is None and city_country is None :
        raise ValueError("Please provide GPS coordinates or a 'city_country' string.")
    elif latitude is not None and longitude is not None and city_country is not None :
        raise ValueError("Please provide EITHER GPS coordinates or a 'city_country' string.")


    # From frontend get longitude, latitude and prefence list
    if city_country is None :
        coordinates = tuple((latitude, longitude))
    
    
    [], landmarks_short = generate_landmarks(preferences=preferences, city_country=city_country, coordinates=coordinates)

    start = Landmark(name='start', type=LandmarkType(landmark_type='start'), location=(48.8375946, 2.2949904), osm_type='start', osm_id=0, attractiveness=0, must_do=True, n_tags = 0)
    finish = Landmark(name='finish', type=LandmarkType(landmark_type='finish'), location=(48.8375946, 2.2949904), osm_type='finish', osm_id=0, attractiveness=0, must_do=True, n_tags = 0)

    landmarks_short.insert(0, start)
    landmarks_short.append(finish)

    max_walking_time = 4 # hours

    visiting_list = solve_optimization(landmarks_short, max_walking_time*60, True)

    return visiting_list



# input city, country in the form of 'Paris, France'
@app.post("/test2/{city_country}")
def test2(city_country: str, preferences: Preferences = Body(...)) -> List[Landmark]:

    landmarks = generate_landmarks(city_country, preferences)

    max_steps = 9000000

    visiting_order = solve_optimization(landmarks, max_steps, True)




