from shapely import buffer, LineString, Point, Polygon
from typing import List
from math import pi

from structs.landmarks import Landmark
from landmarks_manager import take_most_important


def create_corridor(landmarks: List[Landmark], width: float) :

  corrected_width = (180*width)/(6371000*pi)
  
  path = create_linestring(landmarks)
  obj = buffer(path, corrected_width, join_style="mitre", cap_style="square", mitre_limit=2)

  return obj


def create_linestring(landmarks: List[Landmark])->List[Point] :

  points = []

  for landmark in landmarks :
    points.append(Point(landmark.location))

  return LineString(points)


def is_in_area(area: Polygon, coordinates) -> bool :

  point = Point(coordinates)
  return point.within(area)

def get_minor_landmarks(all_landmarks: List[Landmark], visited_landmarks: List[Landmark], width: float) -> List[Landmark] :

  second_order_landmarks = []
  area = create_corridor(visited_landmarks, width)

  for landmark in all_landmarks :
    if is_in_area(area, landmark.location) and landmark not in visited_landmarks:
      second_order_landmarks.append(landmark)

  return take_most_important(second_order_landmarks)